//
//  DebugInfoController.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/8/31.
//

import UIKit

private let K_CELL = "cellID"
class DebugInfoController : UITableViewController {
    private var dataSourceWithDetailText = [String: String]();
    
    private func convertToJsonData(_ dict: [String : Any]) -> String{
        let jsonData = try? JSONSerialization.data(withJSONObject: dict, options: .prettyPrinted);
        var jsonString = String(data: jsonData ?? Data(), encoding: .utf8);
        jsonString = jsonString?.replacingOccurrences(of: " ", with: "");
        jsonString = jsonString?.replacingOccurrences(of: "\n", with: "");
        jsonString = jsonString?.replacingOccurrences(of: "\\/\\/", with: "//");
        return jsonString ?? "";
    }
    
    override func viewDidLoad() {
        super.viewDidLoad();
        self.view.backgroundColor = .white;
        self.title = "DebugInfo";
        self.initDataSource();
        self.setupNavigationItems();
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated);
        self.tableView.reloadData();
    }
    
    private func initDataSource(){
        self.dataSourceWithDetailText["BundleID"] = Bundle.main.bundleIdentifier;
        
        let debugInfo = TMFMiniAppSDKManager.sharedInstance().getDebugInfo() as! [String : Any];
        for (key, value) in debugInfo{
            if (key == "domains" || key == "privacy_apis"){
                self.dataSourceWithDetailText[key] = self.convertToJsonData(value as! Dictionary);
            } else {
                self.dataSourceWithDetailText[key] = value as? String;
            }
        }
        self.dataSourceWithDetailText["DeviceModel"]    = self._UIDevice_mqqIdentifier();
        self.dataSourceWithDetailText["Firmware"]       = self.firmware();
        self.dataSourceWithDetailText["AppVersion"]     = Bundle.main.infoDictionary!["CFBundleShortVersionString"] as? String;
        self.dataSourceWithDetailText["AppBuildNo"]     = Bundle.main.infoDictionary![kCFBundleVersionKey as String] as? String;
    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.dataSourceWithDetailText.count;
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        var cell = tableView.dequeueReusableCell(withIdentifier: K_CELL);
        if (cell == nil){
            cell = UITableViewCell(style: .subtitle, reuseIdentifier: K_CELL);
        }
        let keyName = Array(self.dataSourceWithDetailText.keys)[indexPath.row];
        cell?.textLabel?.text = keyName;
        cell?.detailTextLabel?.text = self.dataSourceWithDetailText[keyName];
        
        return cell!;
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.deselectRow(at: indexPath, animated: true);
        let title = Array(self.dataSourceWithDetailText.keys)[indexPath.row];
        self.didSelectCell(withTitle: title);
    }
    
    private func didSelectCell(withTitle title: String){
        self.copyInfo(title);
    }
    
    private func copyInfo(_ title: String){
        let info = self.dataSourceWithDetailText[title];
        if (info != nil){
            UIPasteboard.general.string = info;
        }
    }
    
    @objc func openServerConfigList(){
        self.navigationController?.pushViewController(TMFAppletConfigListViewController(), animated: true);
    }
    
    func setupNavigationItems(){
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .bookmarks, target: self, action: #selector(openServerConfigList));
        self.title = "DebugInfo";
    }  
    
    // 从 `UIDevice (MQQExtension)` 迁移
    private func _UIDevice_mqqIdentifier() -> String{
        var size: size_t = 0
        sysctlbyname("hw.machine", nil, &size, nil, 0)
        var machine = [CChar](repeating: 0, count: Int(size))
        sysctlbyname("hw.machine", &machine, &size, nil, 0)
        let identifier = String(cString: machine)
        return identifier;
    }
    
    private func firmware() -> String{
        return UIDevice.current.systemVersion;
    }
}
