//
//  DemoTableCell.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/8/30.
//

import UIKit

protocol DemoTableCellDelegate : NSObject {
    /**
     * Dynamically change the height of UITableViewCell
     */
    func updateTableViewCell(_ cell: DemoTableCell, _ height: CGFloat, _ indexPath: IndexPath);
    
    /**
     * Click on the proxy method of UICollectionViewCell
     */
    func didSelectItem(at indexPath: IndexPath, _ content: String?);
}

private let K_CELL = "cell";
class DemoTableCell : UITableViewCell, UICollectionViewDelegate, UICollectionViewDataSource {
    public weak var delegate: DemoTableCellDelegate?;
    public var indexPath: IndexPath?;
    public var dataAry: Array<TMFMiniAppInfo>? {
        willSet{
            self.heightED = 0;
        }
        didSet{
            self.collectionView.reloadData();
        }
    }
    private var heightED: CGFloat = 0.0;
    
    lazy var collectionView: UICollectionView = {
        let layout = UICollectionViewFlowLayout();
        layout.scrollDirection = .vertical;
        let width = (UIScreen.main.bounds.size.width - 50) / 4;
        layout.itemSize = CGSize(width: width, height: 80);
        layout.sectionInset = UIEdgeInsets(top: 10, left: 10, bottom: 10, right: 10);
        
        let collectionView = UICollectionView(frame: CGRectZero, collectionViewLayout: layout);
        collectionView.delegate = self;
        collectionView.dataSource = self;
        collectionView.backgroundColor = .clear;
        collectionView.register(DemoCollectionCell.self, forCellWithReuseIdentifier: K_CELL);
        return collectionView;
    }()
    
    override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
        super.init(style: style, reuseIdentifier: reuseIdentifier);
        self.contentView.addSubview(self.collectionView);
        self.collectionView.frame = CGRectMake(0, 0, UIScreen.main.bounds.size.width, self.contentView.bounds.size.height);
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    
    // UICollectionViewDelegate
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        if ((self.dataAry?.count ?? 0) == 0) {
            return 1;
        } else {
            return self.dataAry!.count;
        }
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: K_CELL, for: indexPath) as! DemoCollectionCell;
        cell.appInfo = self.dataAry?[indexPath.row];
        self.updateCollectionView(self.collectionView.collectionViewLayout.collectionViewContentSize.height);
        return cell;
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        self.delegate?.didSelectItem(at: indexPath, self.dataAry?[indexPath.row].appId);
    }
    
    private func updateCollectionView(_ height: CGFloat){
        if (height == self.heightED){
            return ;
        }
        
        self.heightED = height;
        self.collectionView.frame = CGRectMake(0, 0, self.collectionView.frame.size.width, height);

        self.delegate?.updateTableViewCell(self, height, self.indexPath!);
    }
}

