//
//  MIniAppDemoSDKDelegateImpl.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/8/30.
//

import Foundation

class MIniAppDemoSDKDelegateImpl : NSObject, TMFMiniAppSDKDelegate {
    
    static public let shared = MIniAppDemoSDKDelegateImpl();
    
    func log(_ level: MALogLevel, msg: String) {
        var strLevel = "Undef";
        switch (level){
        case .error:  strLevel = "Error"; break;
        case .warn:   strLevel = "Warn";  break;
        case .info:   strLevel = "Info";  break;
        case .debug:  strLevel = "Debug"; break;
        default: break;
        }
        NSLog("TMFMiniApp %@|%@", strLevel, msg);
    }
    
    // TMFMiniAppSDKDelegate
    func appName() -> String {
        return "TCMPP";
    }
    
    func fetchAppUserInfo(withScope scope: String, block: @escaping TMAAppFetchUserInfoBlock) {
        let filePath = Bundle.main.resourcePath?.appending("/avatar.png");
        let defaultAvatar = UIImage(contentsOfFile: filePath!);
        let avatarView = UIImageView(image: defaultAvatar);
        let userInfo = TMAAppUserInfo();
        userInfo.avatarView = avatarView;
        userInfo.nickName = DemoUserInfo.shared.nickName!;
        block(userInfo);
    }
    
    func getAppUID() -> String {
        return DemoUserInfo.shared.nickName!;
    }
    
    func shareMessage(with shareModel: TMAShareModel, appInfo: TMFMiniAppInfo, completionBlock: ((Error?) -> Void)? = nil) {
        NSLog("shareMessageWithModel \(shareModel.config.shareTarget)");

    }
    
    func getUserProfile(_ app: TMFMiniAppInfo, params: [AnyHashable : Any], completionHandler: @escaping MACommonCallback) {
        let userInfo = ["nickName": DemoUserInfo.shared.nickName!,
                        "avatarUrl": DemoUserInfo.shared.avatarUrl!,
                        "gender": DemoUserInfo.shared.gender!,
                        "country": DemoUserInfo.shared.country!,
                        "province": DemoUserInfo.shared.province!,
                        "city": DemoUserInfo.shared.city!,
                        "language": "zh_CN"] as [String : Any];
        
        completionHandler(userInfo, nil);
    }
    
    func getUserInfo(_ app: TMFMiniAppInfo, params: [AnyHashable : Any], completionHandler: @escaping MACommonCallback) {
        let userInfo = ["nickName": DemoUserInfo.shared.nickName!,
                        "avatarUrl": DemoUserInfo.shared.avatarUrl!,
                        "gender": DemoUserInfo.shared.gender!,
                        "country": DemoUserInfo.shared.country!,
                        "province": DemoUserInfo.shared.province!,
                        "city": DemoUserInfo.shared.city!,
                        "language": "zh_CN"] as [String : Any];
        
        completionHandler(userInfo, nil);
    }

    func handleStartUpSuccess(withApp app: TMFMiniAppInfo) {
        NSLog("handleStartUpSuccess \(app)", app);
        NotificationCenter.default.post(name: Notification.Name(TMF_APPLET_LIST_CHANGE_NOTIFICATION), object: nil);
    }
    
    func handleStartUpError(_ error: Error, app: String?, parentVC: UIViewController) {
        NSLog("handleStartUpError \(String(describing: app)) \(error)");
    }
    
    func uploadLogFile(withAppID appID: String) {
        var path = TMFMiniAppSDKManager.sharedInstance().sandBoxPath(withAppID: appID);
        path.append("/usr/miniprogramLog/");
        NSLog(path);
    }
    
    func vConsolEnabled() -> Bool {
        return true;
    }
    
    func inspectableEnabled() -> Bool {
        return true;
    }
}
