//
//  MiniAppTableCell.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/8/30.
//

import UIKit

class MiniAppTableCell : UITableViewCell {
    private var _appInfo : TMFMiniAppInfo?;
    
    private var titleLabel : UILabel?;
    private var iconView : UIImageView?;
    private var verTypeLabel : UILabel?;

    override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
        super.init(style: style, reuseIdentifier: reuseIdentifier);
        
        self.backgroundColor = .white;
        self.iconView = UIImageView();
        self.contentView.addSubview(self.iconView!);
        
        self.titleLabel = UILabel();
        self.titleLabel?.textAlignment = .left;
        self.titleLabel?.font = UIFont.systemFont(ofSize: 14);
        self.contentView.addSubview(self.titleLabel!);
        
        self.verTypeLabel = UILabel();
        self.verTypeLabel?.backgroundColor = UIColor(red: 0.9176, green: 0.9451, blue: 0.9922, alpha: 1);
        self.verTypeLabel?.font = UIFont.systemFont(ofSize: 12);
        self.verTypeLabel?.textColor = UIColor(red: 0.157, green: 0.443, blue: 0.965, alpha: 1);
        self.verTypeLabel?.textAlignment = .center;
        self.verTypeLabel?.layer.cornerRadius = 10;
        self.verTypeLabel?.layer.masksToBounds = true;
        self.verTypeLabel?.isHidden = true;
        self.contentView .addSubview(self.verTypeLabel!);
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    public var appInfo : TMFMiniAppInfo? {
        get{ return _appInfo }
        set{
            if (_appInfo == newValue){
                return;
            }
            
            _appInfo = newValue;
            self.titleLabel?.text = _appInfo!.appTitle;
            
            switch (_appInfo!.verType) {
            case .develop:
                self.verTypeLabel?.isHidden = false;
                self.verTypeLabel?.text = NSLocalizedString("Develop", comment: "");
            case .preview:
                self.verTypeLabel?.isHidden = false;
                self.verTypeLabel?.text = NSLocalizedString("Preview", comment: "");
            case .audit:
                self.verTypeLabel?.isHidden = false;
                self.verTypeLabel?.text = NSLocalizedString("Reviewed", comment: "");
            default:
                self.verTypeLabel?.isHidden = true;
            }
            
            if (((_appInfo?.appIcon)) != nil){
                DispatchQueue.global(qos: .userInitiated).async {
                    let data = NSData(contentsOf: URL(string: self.appInfo!.appIcon)!);
                    if ((data) != nil){
                        DispatchQueue.main.async {
                            self.iconView?.image = UIImage(data: data! as Data);
                        }
                    } else {
                        DispatchQueue.main.async {
                            self.iconView?.image = UIImage(named: "tmf_weapp_icon_default");
                        }
                    }
                }
            } else {
                self.iconView?.image = UIImage(named: "tmf_weapp_icon_default");
            }
        }
    }
        
    override var frame: CGRect {
        didSet{
            var newFrame = frame;
            newFrame.origin.y += 10;
            newFrame.size.height -= 10;
            super.frame = newFrame;
        }
    }
    
    override func layoutSubviews() {
        super.layoutSubviews();
        
        self.iconView?.frame = CGRectMake(10, 15, 48, 48);
        self.titleLabel?.frame = CGRectMake(70, 0, self.frame.size.width - 138, 78);
        self.verTypeLabel?.frame = CGRectMake(self.frame.size.width - 80, 27, 60, 24);
    }
}
