//
//  TMFAppletConfigInputViewController.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/11/29.
//

import UIKit

typealias AddConfigHandler = (Error?) -> Void
class TMFAppletConfigInputViewController: UIViewController, UITextViewDelegate {
    var addHandler: AddConfigHandler?;
    
    private var textView = UITextView();
    private var textViewMinimumHeight: Float = 0.0;
    private var toolBar = UIToolbar();
    private var tipsLabel = UILabel();
    private var textField = UITextField();
    private var placeholderLabel = UILabel();
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.view.backgroundColor = .white;

        self.view.addSubview(self.toolBar);
        let cancelBtn = UIBarButtonItem(title: NSLocalizedString("Cancel", comment: "Cancel"), style: .plain, target: self, action: #selector(clickCancel));
        let fix = UIBarButtonItem(barButtonSystemItem: .flexibleSpace, target: nil, action: nil);
        let confirmBtn = UIBarButtonItem(title: NSLocalizedString("Save", comment: "Save"), style: .plain, target: self, action: #selector(clickConfirm));
        self.toolBar.setItems([cancelBtn, fix, confirmBtn], animated: false);
        
        let stringAttributes = [NSAttributedString.Key.font: UIFont.systemFont(ofSize: 12),
                                NSAttributedString.Key.foregroundColor: UIColor(red: 153/255.0, green: 160/255.0, blue: 170/255.0, alpha: 1.0),
                                NSAttributedString.Key.paragraphStyle: NSMutableParagraphStyle.default];
        self.tipsLabel.attributedText = NSAttributedString(string: NSLocalizedString("Please enter a profile name:", comment: ""), attributes: stringAttributes);
        self.tipsLabel.numberOfLines = 0;
        self.view.addSubview(self.tipsLabel);
        
        self.textField.font = UIFont.systemFont(ofSize: 16);
        self.textField.layer.cornerRadius = 2;
        self.textField.layer.borderColor = UIColor(red: 222/255.0, green: 224/255.0, blue: 226/255.0, alpha: 1.0).cgColor;
        self.textField.layer.borderWidth = 1;
        
        let paragraphStyle = NSMutableParagraphStyle();
        paragraphStyle.firstLineHeadIndent = 10;
        let placeholder = NSAttributedString(string: NSLocalizedString("Configuration information name", comment: ""),
                                             attributes: [NSAttributedString.Key.foregroundColor: UIColor.lightGray,
                                                          NSAttributedString.Key.paragraphStyle: paragraphStyle]);
        self.textField.attributedPlaceholder = placeholder;
        self.textField.clearButtonMode = .always;
        self.view.addSubview(self.textField);
        
        self.textView.delegate = self;
        self.textView.textContainerInset = UIEdgeInsets(top: 10, left: 7, bottom: 10, right: 7);
        self.textView.returnKeyType = .send;
        
        self.placeholderLabel.text = NSLocalizedString("Configuration information content can be pasted here by long pressing", comment: "");
        self.placeholderLabel.textColor = .lightGray;
        self.placeholderLabel.font = self.textView.font;
        self.placeholderLabel.numberOfLines = 3;
        self.placeholderLabel.sizeToFit();
        self.textView.addSubview(self.placeholderLabel);
        
        self.textView.delegate = self;
        self.textView.layer.borderWidth = 1;
        self.textView.layer.borderColor = UIColor(red: 222/255.0, green: 224/255.0, blue: 226/255.0, alpha: 1.0).cgColor;
        self.textView.layer.cornerRadius = 4;
        self.view.addSubview(self.textView);
        
        // Do any additional setup after loading the view.
    }
    
    func textViewDidChange(_ textView: UITextView) {
        if (self.textView.text.count > 0){
            self.placeholderLabel.isHidden = true;
        } else {
            self.placeholderLabel.isHidden = false;
        }
    }
                   
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews();
        
        let height = self.view.bounds.height;
        var y = 0;
        
        self.toolBar.frame = CGRect(x: 0, y: 0, width: self.view.bounds.width, height: 44);
        y += 40;
        
        let contentWidth = self.view.bounds.width - 32;
        let tipsLabelHeight = 40;
        self.tipsLabel.frame = CGRect(x: 16, y: y, width: Int(contentWidth), height: tipsLabelHeight);
        y += tipsLabelHeight + 5;
        
        self.textField.frame = CGRect(x: 16, y: y, width: Int(contentWidth), height: 40);
        y += 45;
        
        self.textView.frame = CGRect(x: 16, y: y, width: Int(contentWidth), height: Int(height) - y - 50);
        
        self.placeholderLabel.frame = CGRect(x: 16, y: y, width: Int(contentWidth), height: 60);
    }
    
    func shouldHideKeyboardWhenTouch(_ view: UIView) -> Bool {
        return true;
    }
             
    /// Slider function
    @objc func clickCancel(){
        self.dismiss(animated: true);
    }
    
    @objc func clickConfirm(){
        let title = self.textField.text;
        let content = self.textView.text;
        
        if (title?.count ?? 0 <= 0 || content?.count ?? 0 <= 0) {
            self.showError(NSLocalizedString("The input content is empty, unable to add!", comment: ""));
        } else if (title?.count ?? 0 < 2 || title?.count ?? 0 > 10){
            self.showError(NSLocalizedString("Please enter 2~10 characters for the server name!", comment: ""));
        } else if (TMFAppletConfigManager.shared.checkAppletConfigTitle(title!)) {
            self.showError(NSLocalizedString("Server name already exists!", comment: ""));
        } else {
            if (TMFAppletConfigManager.shared.addAppletConfig(title!, content)) {
                if (self.addHandler != nil) {
                    (self.addHandler!)(nil);
                }
                self.dismiss(animated: true);
            } else {
                self.showError(NSLocalizedString("Configuration file information error!", comment: ""));
            }
        }
    }

    /// Helper
    func showError(_ msg: String){
        let alert = UIAlertController(title: "Error", message: msg, preferredStyle: .alert);
        self.present(alert, animated: true);
        DispatchQueue.main.asyncAfter(deadline: .now() + 3) {
            alert.dismiss(animated: true);
        }
    }
}
