//
//  TMFAppletConfigListViewController.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/11/29.
//

import UIKit

class TMFAppletConfigListViewController: UITableViewController  {

    private func configList() -> [TMFAppletConfigItem] {
        return TMFAppletConfigManager.shared.configList;
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.setupNavigationItems();
        // Do any additional setup after loading the view.
    }
    
    func setupNavigationItems() {
        self.title = NSLocalizedString("configure server", comment: "");
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .add, target: self, action: #selector(handleExitEvent));
    }
    
    @objc func handleExitEvent() {
        let inputViewController = TMFAppletConfigInputViewController()
        inputViewController.addHandler = { [weak self] err in
            self?.tableView.reloadData();
            
            if let count = self?.configList().count, count == 1 {
                self?.updateSelConfig();
            }
        }
        present(inputViewController, animated: true, completion: nil);
    }
    
    /// Datasource & Delegate
    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return self.configList().count > 0 ? NSLocalizedString("Please select from the list below or add through the upper right corner", comment: "") : NSLocalizedString("There is no configuration information at present, please add it in the upper right corner", comment: "");

    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.configList().count;
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let identifier = "cell";
        var cell = tableView.dequeueReusableCell(withIdentifier: identifier);
        if cell == nil {
            cell = UITableViewCell(style: .subtitle, reuseIdentifier: identifier);
        }
        
        let row = indexPath.row;
        
        cell?.textLabel?.text = self.configList()[row].title;
        cell?.detailTextLabel?.text = self.configList()[row].subTitle;
        cell?.accessoryType = self.configList()[row].checkmark ? .checkmark : .none;
        
        return cell!;
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.deselectRow(at: indexPath, animated: true);
        let row = indexPath.row;
        let configItem = self.configList()[row];
        if configItem.checkmark {
            return;
        }
        
        configItem.changeCheckmark(true);
        
        var indexPathsAnimated = [IndexPath]();
        for i in 0..<tableView.numberOfRows(inSection: 0) {
            if i != row {
                indexPathsAnimated.append(IndexPath(row: i, section: 0));
                self.configList()[i].changeCheckmark(false);
            }
        }
        
        self.tableView.reloadRows(at: indexPathsAnimated, with: .none);
        
        let cell = tableView.cellForRow(at: indexPath);
        cell?.accessoryType = .checkmark;
        
        self.updateSelConfig();
    }
    
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        return !self.configList()[indexPath.row].checkmark;
    }
    
    override func tableView(_ tableView: UITableView, editingStyleForRowAt indexPath: IndexPath) -> UITableViewCell.EditingStyle {
        return .delete;
    }
    
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCell.EditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            TMFAppletConfigManager.shared.removeAppletConfigItem(self.configList()[indexPath.row].filePath!);
            tableView.deleteRows(at: [indexPath], with: .fade);
        }
    }
    
    override func tableView(_ tableView: UITableView, titleForDeleteConfirmationButtonForRowAt indexPath: IndexPath) -> String? {
        return "delete";
    }
    
    func getUDID() -> String{
        guard var udid = UserDefaults.standard.object(forKey: "udid") else {
            let timeStemp = String(format: "test%.0f", Date.now.timeIntervalSince1970);
            UserDefaults.standard.setValue(timeStemp, forKey: "udid");
            return timeStemp;
        }
        return udid as! String;
    }
    
    func updateSelConfig(){
        guard let configItem = TMFAppletConfigManager.shared.getCurrentConfigItem() else {
            return;
        }
        
        let serverConfig = TMAServerConfig(sting: configItem.content!);
        serverConfig.customizedUDID = self.getUDID();
        TMFMiniAppSDKManager.sharedInstance().setConfiguration(serverConfig);
    }
}
