//
//  ViewController.swift
//  TCMPPDemo-Swift
//
//  Created by v_zwtzzhou on 2023/8/30.
//

import UIKit

let TMF_APPLET_LIST_CHANGE_NOTIFICATION = "com.tencent.tcmpp.apps.change.notification"
private let K_T_CELL = "t_cell"
private let K_C_CELL = "c_cell"
class ViewController: UIViewController, TMFCodeScannerControllerDelegate,  UITableViewDelegate, UITableViewDataSource, DemoTableCellDelegate, UISearchResultsUpdating{

    private lazy var demoList = [TMFMiniAppInfo]();
    private lazy var recentList = [TMFMiniAppInfo]();
    private var navButton: UIButton?;
    private lazy var dicH = [IndexPath: CGFloat]();
    private var searchController: UISearchController?;

    lazy var scanButton: UIButton = {
        let titleLabel = UILabel();
        titleLabel.text = NSLocalizedString("Scan the QR code of the miniapp", comment: "");
        titleLabel.textColor = .white;
        
        let labelSize = titleLabel.sizeThatFits(CGSizeMake(self.view.frame.size.width - 120, 50));
        let buttonWidth = labelSize.width + 100;
        
        let scanButton = UIButton(type: .custom);
        scanButton.frame = CGRectMake((self.view.frame.size.width - buttonWidth) / 2.0,
                                      self.view.frame.size.height - 80,
                                      buttonWidth, 50);
        // create gradient layer
        let gradientLayer = CAGradientLayer();
        gradientLayer.frame = scanButton.bounds;
        // Set gradient color direction
        gradientLayer.startPoint = CGPointZero;
        gradientLayer.endPoint = CGPointMake(1, 0);
        gradientLayer.colors = [UIColor(red: 0, green: 0.431, blue: 1, alpha: 1).cgColor, UIColor(red: 0, green: 0.431, blue: 1, alpha: 0.4).cgColor];
        // Add the gradient layer to the button's layer
        scanButton.layer.addSublayer(gradientLayer);
        scanButton.layer.cornerRadius = 20;
        scanButton.layer.masksToBounds = true;
        
        titleLabel.frame = CGRectMake((buttonWidth - labelSize.width - 25) / 2 + 25, (50 - labelSize.height) / 2, labelSize.width, labelSize.height);
        scanButton.addSubview(titleLabel);
        
        let imageView = UIImageView(frame: CGRectMake(titleLabel.frame.origin.x - 25, 15, 20, 20));
        imageView.image = UIImage(named: "tmf_scan_icon_default");
        scanButton.addSubview(imageView);
        
        scanButton.addTarget(self, action: #selector(openQRCodeController), for: .touchUpInside);
        return scanButton;
    }();
    
    lazy var tableView: UITableView = {
        let frame = CGRectMake(10, 0, self.view.frame.size.width - 20, self.view.frame.size.height - 80);
        let tableView = UITableView(frame: frame, style: .plain);
        tableView.backgroundColor = UIColor.hexColor(0xE3EDFD);
        tableView.separatorStyle = .none;
        tableView.delegate = self;
        tableView.dataSource = self;
        tableView.sectionHeaderHeight = 20;
        return tableView;
    }();

    
    
    static func setNavigationBar(_ navigationBar: UINavigationBar, backgroundColor color: UIColor){
        if #available(iOS 15.0, *) {
            let appearance = UINavigationBarAppearance();
            appearance.configureWithOpaqueBackground();
            appearance.backgroundColor = color;
            appearance.backgroundImage = nil;
            appearance.titleTextAttributes = [NSAttributedString.Key.foregroundColor: UIColor.black];
            
            navigationBar.standardAppearance = appearance;
            navigationBar.scrollEdgeAppearance = appearance;
        } else {
            let image = navigationBar.backgroundImage(for: .default);
            if (image != nil){
                navigationBar.setBackgroundImage(nil, for: .default);
            }
            navigationBar.shadowImage = UIImage();
            navigationBar.barTintColor = color;
            navigationBar.titleTextAttributes = [NSAttributedString.Key.foregroundColor: UIColor.black];
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view.
        
        self.view.backgroundColor = UIColor.hexColor(0xE3EDFD);
        if (self.navigationController != nil){
            ViewController.setNavigationBar(self.navigationController!.navigationBar, backgroundColor: .clear);
        }
        
        // for debug,show the debug info
        let img = UIImage.createImage(with: .clear, andSize: CGSize(width: 48, height: 48), 0);
        let leftItem = UIBarButtonItem(image: img, style: .plain, target: self, action: #selector(openDebugInfoController));
        self.navigationItem.setLeftBarButton(leftItem, animated: true);
        
        self.navigationItem.title = NSLocalizedString("MiniApp Assistant", comment: "");
        
        self.view.addSubview(self.tableView);
        self.view.addSubview(self.scanButton);
        
        self.initDataSource();
        
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(appletListChange),
                                               name: Notification.Name( TMF_APPLET_LIST_CHANGE_NOTIFICATION),
                                               object: nil);
        self.tableView.reloadData();
        
        self.searchController = UISearchController(searchResultsController: SearchResultViewController());
        self.searchController?.searchResultsUpdater = self;
        self.searchController?.obscuresBackgroundDuringPresentation = false;
        self.navigationItem.searchController = self.searchController;
        self.definesPresentationContext = true;
    }
    
    private func initDataSource(){
        let list = TMFMiniAppSDKManager.sharedInstance().loadAppletsFromCache() as? [TMFMiniAppInfo];
        if (list?.count ?? 0 > 0){
            self.recentList.removeAll();
            self.recentList += list!;
        }
        
        self.demoList.removeAll();
        
        let filePath = Bundle.main.path(forResource: "default_mini_apps", ofType: "json");
        if (filePath == nil){
            return;
        }
        let fileData = NSData(contentsOfFile: filePath!) as? Data;
        if (fileData == nil){
            return;
        }
        
        let apps = try? JSONSerialization.jsonObject(with: fileData!) as? [[String : Any]];
        if (apps == nil) {
            return;
        }

        for app in apps!{
            let appId = app["appId"] as? String;
            if (appId?.count ?? 0 > 0){
                let mAppI = TMFMiniAppInfo();
                mAppI.appTitle = app["name"] as! String;
                mAppI.appId = appId!;
                mAppI.appIcon = app["iconurl"] as! String;
                mAppI.verType = .online;
                
                self.addDemoList(mAppI);
            }
        }
    }
    
    private func addDemoList(_ appInfo: TMFMiniAppInfo){
        for app in self.recentList{
            if ((app.appId == appInfo.appId) && (app.verType == appInfo.verType)) {
                self.demoList.append(app);
                return;
            }
        }
        
        self.demoList.append(appInfo);
    }
    
    @objc private func appletListChange(){
        self.initDataSource();
        self.tableView.reloadData();
    }

    @objc private func openQRCodeController(){
        let viewController = TMFCodeScannerController();
        viewController.delegate = self;
        self.navigationController?.pushViewController(viewController, animated: true);
    }
    
    @objc func openDebugInfoController(){
        let viewController = DebugInfoController();
        self.navigationController?.pushViewController(DebugInfoController(), animated: true);
    }
    
    private func showErrorInfo(_ error: NSError){
        let localizedDescription = error.userInfo[NSLocalizedDescriptionKey] as? String;
        var errMsg = "";
        if (localizedDescription != nil){
            errMsg = String(format: "%@\n%ld\n%@", error.domain, error.code, localizedDescription!);
        } else {
            errMsg = String(format: "%@\n%ld", error.domain, error.code);
        }
        
        let alert = UIAlertController(title: "Error", message: errMsg, preferredStyle:.alert);
        self.present(alert, animated: true);
        DispatchQueue.main.asyncAfter(deadline: .now() + 2, execute: {
            alert.dismiss(animated: true);
        });
    }
    
    /// TMFCodeScannerControllerDelegate
    func codeScannerControllerDidCancel() {
        self.navigationController?.popViewController(animated: true);
    }
    
    func codeScannerController(_ type: TMFCodeScannerResource, didScanResult result: [TMFCodeDetectorResult]) {
        if (result.count <= 0){
            return ;
        }
        
        let qrData = result[0].data;
        let tempVCA = self.navigationController!.viewControllers as! NSMutableArray ;
        for tempVC in tempVCA{
            if ((tempVC as AnyObject).isKind(of: TMFCodeScannerController.self)){
                tempVCA.remove(tempVC);
                break;
            }
        }
        
        self.navigationController!.viewControllers = tempVCA as! [UIViewController];
        
        TMFMiniAppSDKManager.sharedInstance().startUpMiniApp(withQrData: qrData, parentVC: self) { error in
            if (error != nil){
                self.showErrorInfo(error! as NSError);
            }
        }
    }
    
    /// UITableViewDelegate
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2;
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return (0 == section ? 1 : self.recentList.count);
    }
    
    func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        if (section > 0 && self.recentList.count <= 0){
            return 300;
        } else {
            return CGFLOAT_MIN;
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return self.dicH[indexPath] ?? 88;
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if (0 == indexPath.section){
            var cell = tableView.dequeueReusableCell(withIdentifier: K_C_CELL) as? DemoTableCell;
            if (cell == nil) {
                cell = DemoTableCell(style: .default, reuseIdentifier: K_C_CELL)
            }
            cell?.delegate = self;
            cell?.indexPath = indexPath;
            cell?.dataAry = self.demoList;
            return cell!;
        } else {
            var cell = tableView.dequeueReusableCell(withIdentifier: K_T_CELL) as? MiniAppTableCell;
            if (cell == nil) {
                cell = MiniAppTableCell(style: .default, reuseIdentifier: K_T_CELL)
            }
            cell?.appInfo = self.recentList[indexPath.row];
            return cell!;
        }
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        if (0 == indexPath.section) {
            cell.backgroundColor = .clear;
            return;
        }
        
        let bezierPath = UIBezierPath(roundedRect: cell.bounds, byRoundingCorners: .allCorners, cornerRadii: CGSizeMake(8, 8));
        let layer = CAShapeLayer();
        layer.path = bezierPath.cgPath;
        cell.layer.mask = layer;
    }
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let view = UIView(frame: CGRectMake(10, 0, self.view.bounds.size.width - 20, 20));
        view.backgroundColor = .clear;
        
        let label = UILabel(frame: CGRectMake(10, 0, view.frame.size.width - 10, 20));
        label.font = UIFont.systemFont(ofSize: 12);
        label.textColor = UIColor.hexColor(0x334c6a);
        if (section == 0){
            label.text = NSLocalizedString("MiniApp demo", comment: "");
        } else {
            label.text = NSLocalizedString("Recently used", comment: "");
        }
        view.addSubview(label);
        return view;
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        if (section == 0 || self.recentList.count > 0) {
            return nil;
        }
        
        let view = UIView(frame: CGRectMake(10, 0, self.view.bounds.size.width - 20, 300));
        view.backgroundColor = .clear;
        
        let imageView = UIImageView(frame: CGRectMake((view.frame.size.width - 79) / 2.0, 80, 79, 92));
        imageView.image = UIImage(named: "no_recent_info");
        view.addSubview(imageView);
        
        let titleLabel = UILabel(frame: CGRectMake(0, 180, view.frame.size.width, 20));
        titleLabel.font = UIFont.systemFont(ofSize: 14);
        titleLabel.textAlignment = .center;
        titleLabel.textColor = UIColor(white: 0.2, alpha: 1);
        titleLabel.text = NSLocalizedString("No recently used miniapps", comment: "");
        view.addSubview(titleLabel);
        
        let subTitleLabel = UILabel(frame: CGRectMake(0, 200, view.frame.size.width, 20));
        subTitleLabel.font = UIFont.systemFont(ofSize: 14);
        subTitleLabel.textAlignment = .center;
        subTitleLabel.textColor = UIColor(red: 0.503, green: 0.546, blue: 0.6, alpha: 1);
        subTitleLabel.text = NSLocalizedString("Please scan the QR code of the miniapp to experience", comment: "");
        view.addSubview(subTitleLabel);
        
        return view;
    }
    
    func tableView(_ tableView: UITableView, willDisplayHeaderView view: UIView, forSection section: Int) {
        let bezierPath = UIBezierPath(roundedRect: view.bounds, byRoundingCorners: .allCorners, cornerRadii: CGSizeMake(8, 8));
        let layer = CAShapeLayer();
        layer.path = bezierPath.cgPath;
        view.layer.mask = layer;
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if (1 == indexPath.section){
            let appInfo = self.recentList[indexPath.row];

            TMFMiniAppSDKManager.sharedInstance().startUpMiniApp(withAppID: appInfo.appId,
                                                                 verType: appInfo.verType,
                                                                 scene: .aioEntry,
                                                                 firstPage: nil,
                                                                 paramsStr: nil,
                                                                 parentVC: self) { error in
//                NSLog("open applet error:\(String(describing: error))");
                self.showErrorInfo(error! as NSError);
            }
        }
        
        tableView.deselectRow(at: indexPath, animated: true);
    }
    
    /// DemoTableCellDelegate
    func updateTableViewCell(_ cell: DemoTableCell, _ height: CGFloat, _ indexPath: IndexPath) {
        if(self.dicH[indexPath] != height){
            self.dicH[indexPath] = height;
            self.tableView.reloadData();
        }
    }
    
    func didSelectItem(at indexPath: IndexPath, _ content: String?) {
        TMFMiniAppSDKManager.sharedInstance().startUpMiniApp(withAppID: content!,
                                                             verType: .online,
                                                             scene: .aioEntry,
                                                             firstPage: nil,
                                                             paramsStr: nil,
                                                             parentVC: self) { error in
//            NSLog("open applet error:\(String(describing: error))");
            self.showErrorInfo(error! as NSError);
        }
    }
    
    
    /// Search
    func updateSearchResults(for searchController: UISearchController) {
        guard let inputStr = searchController.searchBar.text, inputStr.count >= 2 else { return }

          TMFMiniAppSDKManager.sharedInstance().searchApplets(withName: inputStr) { result, error in
              if error != nil {
//                  self.showErrorInfo(error as NSError);
                  return
              }

              if let resultVC = self.searchController?.searchResultsController as? SearchResultViewController {
                  resultVC.searchResults.removeAll()
                  if let result = result, !result.isEmpty {
                      resultVC.searchResults.append(contentsOf: result)
                  }
                  resultVC.tableView?.reloadData()
              }
          }
    }
}



extension UIColor{
    static func hexColor(_ hex: UInt32) -> UIColor{
        return UIColor(red: CGFloat((hex & 0xFF0000) >> 16) / 255.0,
                       green: CGFloat((hex & 0xFF00) >> 8) / 255.0,
                       blue: CGFloat(hex & 0xFF) / 255.0,
                       alpha: 1);
    }
}


extension UIImage{
    static func createImage(with color: UIColor, andSize size: CGSize,_ radius: CGFloat) -> UIImage?{
        let rect = CGRectMake(0, 0, size.width, size.height);
        UIGraphicsBeginImageContextWithOptions(rect.size, false, UIScreen.main.scale);
        let context = UIGraphicsGetCurrentContext();
        context?.setFillColor(color.cgColor);
        let path = UIBezierPath(roundedRect: rect, cornerRadius: radius);
        context?.addPath(path.cgPath);
        context?.fillPath();
        
        let image = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        return image;
    }
}
